//*----------------------------------------------------------------------------
//*         ATMEL Microcontroller Software Support  -  ROUSSET  -
//*----------------------------------------------------------------------------
//* The software is delivered "AS IS" without warranty or condition of any
//* kind, either express, implied or statutory. This includes without
//* limitation any warranty or condition with respect to merchantability or
//* fitness for any particular purpose, or against the infringements of
//* intellectual property rights of others.
//*----------------------------------------------------------------------------
//* File Name           : main_oneshot.c
//* Object              : Main application written in C
//* Creation            : S.CADENE   01/September/2004
//* Vers                : V1.0
//* Aim : What is the minimum program in order to achieve one-shot conversion ?
//*       This Main program has been done in this goal. Only five steps for that 
//*       are necessary.
//        In this program, the ADC conversion is started by the software.
//*----------------------------------------------------------------------------

// Include Standard files
#include "project.h"
#include "dbgu.h"
#include "float.h"
/* ADC field definition for the Mode Register: Reminder
                       TRGEN    => Selection bewteen Software or hardware start of conversion
                       TRGSEL   => Relevant if the previous field set a Hardware Triggering Mode
                       LOWRES   => 10-bit result if ths bit is cleared 0
                       SLEEP    => normal mode if ths is cleared
                       PRESCAL  => ADCclock = MCK / [(PRESCAL + 1)*2]
                       STARTUP  => Startup Time = [(STARTUP + 1)*8] / ADCclock
                       SHTIM    => Tracking time = (SHTIM + 1) / ADCclock
 */
#define   TRGEN    (0x0)    // Software triggering
#define   TRGSEL   (0x0)    // Without effect in Software triggering
#define   LOWRES   (0x0)    // 10-bit result output format
#define   SLEEP    (0x0)    // Normal Mode (instead of SLEEP Mode)
#define   PRESCAL  (0x4)    // Max value
#define   STARTUP  (0xc)    // This time period must be higher than 20 s
#define   SHTIM    (0x3)    // Must be higher than 3 ADC clock cycles but depends on output
                            // impedance of the analog driver to the ADC input
/* Channel selection */
#define   CHANNEL4  (4)     // Write the targeted channel (Notation: the first channel is 0
                            // and the last is 7)

//#define stdout AT91C_BASE_DBGU


//const unsigned char a @ 0x0010ff00 = 234;

//*--------------------------------------------------------------------------------------
//* Function Name       : Main
//* Object              : Software entry point
//* Input Parameters    : none.
//* Output Parameters   : none.
//*--------------------------------------------------------------------------------------
//unsigned int adc_init()
void adc_init()
{
    //unsigned int result ;
    // Init trace DBGU
    //AT91F_DBGU_Init();
    //AT91F_DBGU_Printk("\n\rADC\n\r");

    /* Clear all previous setting and result */
    AT91F_ADC_SoftReset (AT91C_BASE_ADC);
       
    /* First Step: Set up by using ADC Mode register */
    AT91F_ADC_CfgModeReg (AT91C_BASE_ADC,
                         (SHTIM << 24) | (STARTUP << 16) | (PRESCAL << 8) | 
                         (SLEEP << 5) | (LOWRES <<4) | (TRGSEL << 1) | (TRGEN )) ;

    /* Second Step: Select the active channel */
    AT91F_ADC_EnableChannel (AT91C_BASE_ADC, (1<<CHANNEL4));  // z-6,y-5,x-4
    AT91F_ADC_EnableChannel (AT91C_BASE_ADC, (1<<CHANNEL4+1));  // z-6,y-5,x-4
    AT91F_ADC_EnableChannel (AT91C_BASE_ADC, (1<<CHANNEL4+2));  // z-6,y-5,x-4    
        
    // Third Step: Start the conversion
    //AT91F_ADC_StartConversion (AT91C_BASE_ADC);
        
    // Fourth Step: Waiting Stop Of conversion by pulling
    //while (!((AT91F_ADC_GetStatus (AT91C_BASE_ADC)) & (1<<CHANNEL)));
        
    // Fifth Step: Read the ADC result output
    //result = AT91F_ADC_GetConvertedDataCH6 (AT91C_BASE_ADC);
        
        //return(result);
        
        // Sixth Step: infinit loop
        /*
        long j=0;
        while (1) 
        {
          change_speed();
          printf("ADC:0x%X\n\r",a);
          j++;
          //AT91F_DBGU_Printk("ADC:0x%X\n\r",result);          
        };
        */
}
//* End
unsigned int adc_get4()
{
    unsigned int result ;

    /* Third Step: Start the conversion */
    AT91F_ADC_StartConversion (AT91C_BASE_ADC);

    /* Fourth Step: Waiting Stop Of conversion by pulling */
    while (!((AT91F_ADC_GetStatus (AT91C_BASE_ADC)) & (1<<CHANNEL4)));

    result = AT91F_ADC_GetConvertedDataCH4 (AT91C_BASE_ADC);
    return(result);
}

unsigned int adc_get5()
{
    unsigned int result ;

    /* Third Step: Start the conversion */
    AT91F_ADC_StartConversion (AT91C_BASE_ADC);

    /* Fourth Step: Waiting Stop Of conversion by pulling */
    while (!((AT91F_ADC_GetStatus (AT91C_BASE_ADC)) & (1<<CHANNEL4+1)));

    result = AT91F_ADC_GetConvertedDataCH5 (AT91C_BASE_ADC);
    return(result);
}

unsigned int adc_get6()
{
    unsigned int result ;

    /* Third Step: Start the conversion */
    AT91F_ADC_StartConversion (AT91C_BASE_ADC);

    /* Fourth Step: Waiting Stop Of conversion by pulling */
    while (!((AT91F_ADC_GetStatus (AT91C_BASE_ADC)) & (1<<CHANNEL4+2)));

    result = AT91F_ADC_GetConvertedDataCH6 (AT91C_BASE_ADC);
    return(result);
}

//

    