//*----------------------------------------------------------------------------
//*         ATMEL Microcontroller Software Support  -  ROUSSET  -
//*----------------------------------------------------------------------------
//  DISCLAIMER:  THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR
//  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
//  DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
//  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
//  OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
//  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
//  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
//  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//*----------------------------------------------------------------------------
//* File Name           : main.c
//* Object              : main application written in C
//* Creation            : JPP   16/Sep/2005
//*----------------------------------------------------------------------------

// Include Standard files
#include "project.h"
#include "dbgu.h"
#include "nokia_lcd.h"
#include "lib_twi.h"
#include "adc.h"
#include "cdc_enumerate.h"
#include "argussoft.h"

/* Global variables */
#define      SPEED 		( AT91B_MAIN_OSC /1000)
unsigned int LedSpeed = SPEED*2;// * 20 ;
const int led_mask[AT91B_NB_LEB]= {AT91B_LED1, AT91B_LED2, AT91B_LED3, AT91B_LED4};

#define RTTC_INTERRUPT_LEVEL            0

// ,   ,   

#define PIV_200_MS              186 // 16KHz

//*--------------------------------------------------------------------------------------
//* Function Name       : change_speed
//* Object              : Adjust "LedSpeed" value depending on SW1 and SW2 are pressed or not
//* Input Parameters    : none
//* Output Parameters   : Update of LedSpeed value.
//*--------------------------------------------------------------------------------------
void change_speed ( void )
{//* Begin
/*
    if ( (AT91F_PIO_GetInput(AT91D_BASE_PIO_SW) & AT91B_SW2) == 0 )
    {
        if ( LedSpeed > SPEED ) LedSpeed -=SPEED ;
    }
    if ( (AT91F_PIO_GetInput(AT91D_BASE_PIO_SW) & AT91B_SW3) == 0 )
    {
        if ( LedSpeed < AT91B_MAIN_OSC ) LedSpeed +=SPEED ;
    }
*/  
}//* End

//*--------------------------------------------------------------------------------------
//* Function Name       : wait
//* Object              : Software waiting loop
//* Input Parameters    : none. Waiting time is defined by the global variable LedSpeed.
//* Output Parameters   : none
//*--------------------------------------------------------------------------------------
void wait ( void )
{//* Begin
    unsigned int waiting_time ;
    //change_speed () ;
    for(waiting_time = 0; waiting_time < LedSpeed; waiting_time++) ;
}//* End

//*----------------------------------------------------------------------------
//* Function Name       : Periodic_Interval_Timer_handler
//* Object              : C handler interrupt function called by the interrupts
//*                       assembling routine
//*----------------------------------------------------------------------------

//   
//        16 

void Periodic_Interval_Timer_handler(void)
{
    unsigned int status;
    unsigned char temp;
    static unsigned int page = 0;
    //   wav - 
    static unsigned int byte = 0x30; // skip wav - file header

    //* Interrupt Acknowledge
//    status = AT91C_BASE_PITC->PITC_PIVR;
//        status =status;
//   AT91C_BASE_DBGU->DBGU_THR = '*';
        //* Read the output state
//    if ((AT91F_PIO_GetInput(AT91C_BASE_PIOA) & LED1 ) == LED1 )   AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, LED1 );
//      else  AT91F_PIO_SetOutput( AT91C_BASE_PIOA, LED1 );

  // set LDAC to 1-0-1
  //    LDAC,    

  AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, 1<<9 ) ;
  AT91F_PIO_SetOutput( AT91C_BASE_PIOA, 1<<9 ) ;

  //         

  if(page==4096){page=0;};
  if(byte== 512){byte=0;page++;};

  //        
  temp = AT45_ReadByte(page,byte);

  //  cs  
  AT91F_PIO_ClearOutput( AT91C_BASE_PIOA, 1<<15 ) ;
  
  // send data from dataflash to dac
  SPI_Send(temp << 4, DAC_CS, LAST_XFER);

  //  cs  
  AT91F_PIO_SetOutput( AT91C_BASE_PIOA, 1<<15 ) ;  
  //SPI_Send(0xaa << 4, DAC_CS, LAST_XFER);
  byte++;

  //   
  status = AT91C_BASE_PITC->PITC_PIVR;

  //   /
  status = status;
}

//*--------------------------------------------------------------------------------------
//* Function Name       : Main
//* Object              : Software entry point
//* Input Parameters    : none.
//* Output Parameters   : none.
//*--------------------------------------------------------------------------------------
int main()
{//* Begin
    //int i;
    char Str[100];
    unsigned char strnum = 0;
    RESET_Init();
    //    RS232
    AT91F_DBGU_Init();

    // First, enable the clock of the PIO
    //      PIO

    AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1 << AT91C_ID_PIOA ) ;
    AT91F_PMC_EnablePeriphClock ( AT91C_BASE_PMC, 1 << AT91C_ID_PIOB ) ;

    // then, we configure the PIO Lines corresponding to LED1 to LED4
    // to be outputs. No need to set these pins to be driven by the PIO because it is GPIO pins only.
    //      
    AT91F_PIO_CfgOutput( AT91D_BASE_PIO_LED, AT91B_LED_MASK ) ;

    // Clear the LED's. On the EB55 we must apply a "1" to turn off LEDs

    AT91F_PIO_SetOutput( AT91D_BASE_PIO_LED, AT91B_LED_MASK ) ;

    // Clear the LED's. On the EB55 we must apply a "1" to turn off LEDs

    AT91F_PIO_ClearOutput( AT91D_BASE_PIO_LED, AT91B_LED_MASK ) ;




    AT91F_DBGU_Printk("\n\r.  Argussoft DEMO March/2006)\n\r.  E-mail: m.s.joker@mtu-net.ru");

    LcdInit();


    LcdClear();    
    LcdGotoXY(1,1);//0123456789abcde -----------
    LcdStr (FONT_1X,"Play music");
    LcdGotoXY(1,2);
    LcdStr (FONT_1X,"DBGU->Flash");
    LcdGotoXY(1,3);
    LcdStr (FONT_1X,"USB sample");
    LcdGotoXY(1,4);
    LcdStr (FONT_1X,"ADC internal");
    LcdGotoXY(1,5);
    LcdStr (FONT_1X,"ADXL330-RS232");
    LcdGotoXY(1,6);
    LcdStr (FONT_1X,"TWI");
    LcdInvertStr(strnum);
    LcdUpdate();


    AT91F_DBGU_Printk("\n\r.  Please, select TEST option by pressing SW4(DOWN)/SW3(UP)");
    AT91F_DBGU_Printk("\n\r.  After selecting, press SW1 to start test");
    AT91F_DBGU_Printk("\n\r.  Don`t forget - connect TF & RF pins of AT91 chip for SSC test");
    AT91F_DBGU_Printk("\n\r.  0) Play wav file from flash (soic8)");
    AT91F_DBGU_Printk("\n\r.  1) Transfer wav to flash (soic8) via UART (XModem)");
    AT91F_DBGU_Printk("\n\r.  2) Start ATMEL BasicUSB app note");
    AT91F_DBGU_Printk("\n\r.  3) Start internal ADC to lcd in hex mode");
    AT91F_DBGU_Printk("\n\r.  4) Start ADXL330 programm");
    AT91F_DBGU_Printk("\n\r.  5) Start clock and termometer\n\r");
    AT91F_DBGU_Printk(".  Press SW2(up)/SW3(down)/SW6(select) >0");

    for(;;)
    {


      // up
      //   
      if ( (AT91F_PIO_GetInput(AT91D_BASE_PIO_SW) & AT91B_SW2) == 0 )
      {
        // 3 times needed for delay

        LcdInvertStr(strnum);
        LcdUpdate();
        LcdInvertStr(strnum);
        LcdUpdate();
        LcdInvertStr(strnum);
        LcdUpdate();
        if (strnum == 0){strnum = 5;}else{strnum--;};
        LcdInvertStr(strnum);
        LcdUpdate();
        sprintf(Str,"%d",strnum);
        AT91F_DBGU_Printk(Str);
      };

      // down
      //   
      if ( (AT91F_PIO_GetInput(AT91D_BASE_PIO_SW) & AT91B_SW3) == 0 )
      {
        // 3 times needed for delay
        LcdInvertStr(strnum);
        LcdUpdate();
        LcdInvertStr(strnum);
        LcdUpdate();
        LcdInvertStr(strnum);
        LcdUpdate();
        if (++strnum == 6){strnum = 0;};
        LcdInvertStr(strnum);
        LcdUpdate();
        sprintf(Str,"%d",strnum);
        AT91F_DBGU_Printk(Str);
      }
      
      // Select key
      //      
      if ( (AT91F_PIO_GetInput(AT91D_BASE_PIO_SW) & AT91B_SW6) == 0 )
      {

        //    
        if (strnum == 0)//play music
        {
          //  SPI ( )
          SPI_Init();
          AT91F_DBGU_Printk(".  SPI init done\n\r");
          AT91F_DBGU_Printk(".  playing music now\n\r");
          // __start__ playing wav file from flash memory
          // until timer enabled, music is playing

          //* Init Real Time Timer

          AT91F_AIC_ConfigureIt ( AT91C_BASE_AIC, AT91C_ID_SYS, RTTC_INTERRUPT_LEVEL,AT91C_AIC_SRCTYPE_INT_HIGH_LEVEL, Periodic_Interval_Timer_handler);
          // Set the interrupt mo
          AT91C_BASE_PITC->PITC_PIMR = AT91C_PITC_PITEN | AT91C_PITC_PITIEN | PIV_200_MS;  //  IRQ enable CPC
          //  RTTC_RTAR
          AT91F_AIC_EnableIt (AT91C_BASE_AIC, AT91C_ID_SYS);
          AT91F_DBGU_Printk("\n\r.  RTTC interrupt enabled");

          LcdClear();
          LcdGotoXY(1,1);//0123456789abcde -----------    
          LcdStr (FONT_1X,"Playing music");
          LcdUpdate();
          
          do{}while(1!=0); // 
        };

        
        //   dbgu  wave -  (raw )  
        //     (soic8)
        if (strnum == 1)//xmodem
        {
          LcdClear();
          LcdGotoXY(1,1);//0123456789abcde -----------    
          LcdStr (FONT_1X,"XMODEM started");
          LcdUpdate();

          AT91F_DBGU_Printk(".  XModem started, reboot after sending\n\r");
          SPI_Init();
          XModem();
          
          do{}while(1!=0); // 
        };
        
        //     usb
        if (strnum == 2)//USB interface (Atmel example on PC)
        {
          LcdClear();
          LcdGotoXY(1,1);//0123456789abcde -----------    
          LcdStr (FONT_1X,"USB enum wait");
          LcdUpdate();

          //  USB ,   enumeration    
          //      ,     USB
          USB_Init();
          USB_Start();

          LcdClear();
          LcdGotoXY(1,1);//0123456789abcde -----------    
          LcdStr (FONT_1X,"USB enum ok");
          LcdUpdate();

          USB_Do();
          AT91F_DBGU_Printk(".  USB test started\n\r");
        };
        
        if (strnum == 3)//internal adc, ADXL330
        {
          //unsigned char num,i;
          unsigned int x,y,z;
          //char pkt[16]={0x55,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0xaa};
          
          //num = 0;
          LcdClear();
          adc_init();

          while(1)
          {

            wait();

            x = adc_get4();
            y = adc_get5();
            z = adc_get6();

            sprintf(Str,"x=%8.8x    y=%8.8x    z=%8.8x",x,y,z);
            LcdGotoXY(1,1);
            LcdStr(FONT_1X,(byte*)Str);
            LcdUpdate();
            
            AT91F_DBGU_Printk(Str);
            AT91F_DBGU_Printk("\n\r");
/*
            pkt[1]=num;
            //x
            pkt[9]=x<<2>>8 & 0xff;
            pkt[10]=x<<2 & 0xff;
            //y
            pkt[11]=y<<2>>8 & 0xff;
            pkt[12]=y<<2 & 0xff;
            //z
            pkt[13]=z<<2>>8 & 0xff;
            pkt[14]=z<<2 & 0xff;

            for(i=0;i<16;i++)
            {
              AT91F_DBGU_PutChar(pkt[i]);
            };
            num++;
*/            


          }
        }

        if (strnum == 4)//internal adc, ADXL330
        {
          unsigned char num,i;
          unsigned int x,y,z;
          char pkt[16]={0x55,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0xaa};
          
          num = 0;

          LcdClear();
          LcdGotoXY(1,1);//0123456789abcde -----------    
          LcdStr (FONT_1X,"ADXL330 ok");
          LcdUpdate();

          adc_init();

          while(1)
          {

            wait();

            x = adc_get4();
            y = adc_get5();
            z = adc_get6();

            //sprintf(Str,"x=%8.8x    y=%8.8x    z=%8.8x",x,y,z);
            //LcdGotoXY(1,1);
            //LcdStr(FONT_1X,(byte*)Str);

            //AT91F_DBGU_Printk(Str);
            //AT91F_DBGU_Printk("\n\r");

            pkt[1]=num;
            //x
            pkt[9]=x<<2>>8 & 0xff;
            pkt[10]=x<<2 & 0xff;
            //y
            pkt[11]=y<<2>>8 & 0xff;
            pkt[12]=y<<2 & 0xff;
            //z
            pkt[13]=z<<2>>8 & 0xff;
            pkt[14]=z<<2 & 0xff;

            for(i=0;i<16;i++)
            {
              AT91F_DBGU_PutChar(pkt[i]);
            };
            

            //LcdUpdate();
            num++;
          }
        }
        
        if (strnum == 5)//TWI interface,temperature, clock ds1307
        {
          AT91F_DBGU_Printk(".  Temperature (TWI) test started\n\r");
          //unsigned int address,tmp,
          unsigned int status;
          //char write, read;
          char read2[2];
          char read4[3];
          LcdClear();
          AT91F_TWI_Open();
          
          // start RTC
          read4[0] = 0;
          read4[1] = 0;
          status = AT91F_TWI_WriteByte(AT91C_BASE_TWI, AT91C_TIME_I2C_ADDRESS | AT91C_TWI_IADRSZ_1_BYTE,0, &read4[0], 2);
          
          // print menu
          LcdGotoXY(1,1);
          LcdStr(FONT_1X,"RTC R0-R2");
          LcdGotoXY(1,3);
          LcdStr(FONT_1X,"Temperature");

          do
          {
            // read RTC
            status = AT91F_TWI_ReadByte(AT91C_BASE_TWI, AT91C_TIME_I2C_ADDRESS | AT91C_TWI_IADRSZ_1_BYTE,0, &read4[0], 3);
            // print RTC
            sprintf(Str," Time=%2.2x:%2.2x:%2.2x",read4[2],read4[1],read4[0]);  
            LcdGotoXY(1,2);
            LcdStr(FONT_1X,(byte*)Str);
            AT91F_DBGU_Printk(Str);
            // read temperature
            status = AT91F_TWI_ReadByte(AT91C_BASE_TWI, AT91C_TEMP_I2C_ADDRESS | AT91C_TWI_IADRSZ_1_BYTE,0, &read2[0], 2);
            // print 1/4 of temeperature
            if(read2[1]>>6 == 0)
            {
              sprintf(Str," T=%d.00",(read2[0]));
            }
            if(read2[1]>>6 == 1)
            {
              sprintf(Str," T=%d.25",(read2[0]));
            }
            if(read2[1]>>6 == 2)
            {
              sprintf(Str," T=%d.50",(read2[0]));
            }
            if(read2[1]>>6 == 3)
            {
              sprintf(Str," T=%d.75",(read2[0]));
            }
            // print temeperature
            LcdGotoXY(1,4);
            LcdStr(FONT_1X,(byte*)Str);
            AT91F_DBGU_Printk(Str);
            AT91F_DBGU_Printk("\n\r");

            LcdUpdate();

          }while(1!=0);
        };

        
      }
      
      
    }
      
}      
    // Loop forever
/*
    for (;;)
    {
        // Once a Shot on each led
	for ( i=0 ; i < AT91B_NB_LEB ; i++ )
        {
	    AT91F_PIO_ClearOutput( AT91D_BASE_PIO_LED, led_mask[i]) ;
	    wait();
	    AT91F_PIO_SetOutput( AT91D_BASE_PIO_LED, led_mask[i] ) ;
	    wait();
        }// End for
        // Once a Shot on each led
        for ( i=(AT91B_NB_LEB-1) ; i >= 0 ; i-- )
        {
	    AT91F_PIO_ClearOutput( AT91D_BASE_PIO_LED, led_mask[i]) ;
	    wait();
	    AT91F_PIO_SetOutput( AT91D_BASE_PIO_LED, led_mask[i] ) ;
	    wait();
        }
    }// End for
}// End
*/
    
    

